@@ -12,10 +12,7 @@ __all__ = [
     "SqliteDictFormat",
     "Step",
     "StepCache",
-    "LocalStepCache",
     "Workspace",
-    "MemoryWorkspace",
-    "LocalWorkspace",
 ]
 
 from tango.executor import Executor
@@ -27,8 +24,7 @@ from tango.format import (
     JsonFormatIterator,
     SqliteDictFormat,
 )
-from tango.local_workspace import LocalWorkspace
 from tango.step import Step
-from tango.step_cache import LocalStepCache, StepCache
+from tango.step_cache import StepCache
 from tango.step_graph import StepGraph
-from tango.workspace import MemoryWorkspace, Workspace
+from tango.workspace import Workspace
