@@ -126,6 +126,13 @@ is registered as "pytorch_lightning::EarlyStopping".
 
 """
 
+from tango.common.exceptions import IntegrationMissingError
+
+try:
+    import pytorch_lightning
+except ModuleNotFoundError:
+    raise IntegrationMissingError("pytorch_lightning")
+
 __all__ = [
     "LightningAccelerator",
     "LightningCallback",
