@@ -14,7 +14,7 @@ class LightningProfiler(pl.profiler.base.BaseProfiler, Registrable):
 for name, cls in pl.profiler.__dict__.items():
     if (
         isinstance(cls, type)
-        and issubclass(cls, pl.profiler.base.BaseProfiler)
+        and issubclass(cls, (pl.profiler.base.BaseProfiler, pl.profiler.base.Profiler))
         and not cls == pl.profiler.base.BaseProfiler
     ):
         LightningProfiler.register("pytorch_lightning::" + name)(cls)
